==> json-joy-2.7.0.tgz: package/esm/json-crdt/types/lww-object/ObjectType.js <==
import { UNDEFINED_ID } from '../../../json-crdt-patch/constants';
import { ObjectEntry } from './ObjectEntry';
import { asString } from 'json-schema-serializer';
import { UNDEFINED } from '../../constants';
import { decodeNode } from '../../codec/compact/decodeNode';
export class ObjectType {
    constructor(doc, id) {
        this.doc = doc;
        this.id = id;
        this.latest = new Map();
    }
    get(key) {
        const entry = this.latest.get(key);
        return entry ? entry.value : undefined;
    }
    getId(key) {
        const entry = this.latest.get(key);
        return entry ? entry.id : undefined;
    }
    insert(op) {
        const { latest: last } = this;
        const clock = op.id.clock();
        for (const [key, value] of op.tuples) {
            const id = clock.tick(1);
            const lastEntry = last.get(key);
            if (lastEntry)
                if (lastEntry.id.compare(id) >= 0)
                    continue;
            this.put(key, id, value);
        }
    }
    put(key, id, value) {
        const entry = new ObjectEntry(id, value);
        this.latest.set(key, entry);
    }
    toJson() {
        const obj = {};
        for (const [key, entry] of this.latest.entries()) {
            const { value } = entry;
            if (value.compare(UNDEFINED_ID) === 0)
                continue;
            const node = this.doc.nodes.get(value);
            if (!node)
                continue;
            obj[key] = node.toJson();
        }
        return obj;
    }
    toString(tab = '') {
        let str = `${tab}ObjectType(${this.id.toDisplayString()})`;
        for (const [key, value] of this.latest.entries()) {
            const node = this.doc.nodes.get(value.value) || UNDEFINED;
            str += `\n${tab}  "${key}" :\n${node.toString(tab + '    ')}`;
        }
        return str;
    }
    clone(doc) {
        const obj = new ObjectType(doc, this.id);
        for (const [key, { id, value }] of this.latest.entries())
            obj.put(key, id, value);
        return obj;
    }
    *children() {
        for (const { value } of this.latest.values())
            yield value;
    }
    encodeCompact(codec) {
        let str = '[0,' + codec.encodeTs(this.id);
        for (const [key, value] of this.latest.entries()) {
            const node = this.doc.nodes.get(value.value);
            str += ',' + asString(key) + ',' + codec.encodeTs(value.id) + ',' + node.encodeCompact(codec);
        }
        return str + ']';
    }
    static decodeCompact(doc, codec, data) {
        const length = data.length;
        const objId = codec.decodeTs(data[1], data[2]);
        const obj = new ObjectType(doc, objId);
        for (let i = 3; i < length;) {
            const key = data[i++];
            const id = codec.decodeTs(data[i++], data[i++]);
            const value = decodeNode(doc, codec, data[i++]);
            obj.put(key, id, value.id);
        }
        return obj;
    }
}

==> json-joy-2.8.0.tgz: package/esm/json-crdt/types/lww-object/ObjectType.js <==
import { ObjectChunk } from './ObjectChunk';
export class ObjectType {
    constructor(doc, id) {
        this.doc = doc;
        this.id = id;
        this.latest = new Map();
    }
    get(key) {
        const entry = this.latest.get(key);
        return entry ? entry.node.id : undefined;
    }
    getId(key) {
        const entry = this.latest.get(key);
        return entry ? entry.id : undefined;
    }
    insert(op) {
        const { latest: last } = this;
        const clock = op.id.clock();
        for (const [key, value] of op.tuples) {
            const id = clock.tick(1);
            const lastEntry = last.get(key);
            if (lastEntry)
                if (lastEntry.id.compare(id) >= 0)
                    continue;
            this.put(key, id, value);
        }
    }
    put(key, id, value) {
        const node = this.doc.nodes.get(value);
        if (!node)
            return;
        this.putChunk(key, new ObjectChunk(id, node));
    }
    putChunk(key, chunk) {
        this.latest.set(key, chunk);
    }
    toJson() {
        const obj = {};
        for (const [key, entry] of this.latest.entries())
            obj[key] = entry.node.toJson();
        return obj;
    }
    toString(tab = '') {
        let str = `${tab}ObjectType(${this.id.toDisplayString()})`;
        for (const [key, entry] of this.latest.entries()) {
            const node = entry.node;
            str += `\n${tab}  "${key}" :\n${node.toString(tab + '    ')}`;
        }
        return str;
    }
    clone(doc) {
        const obj = new ObjectType(doc, this.id);
        for (const [key, { id, node }] of this.latest.entries()) {
            const nodeClone = node.clone(doc);
            obj.putChunk(key, new ObjectChunk(id, nodeClone));
        }
        doc.nodes.index(obj);
        return obj;
    }
    *children() {
        for (const { node } of this.latest.values())
            yield node.id;
    }
}
